package de.fitko.fitconnect.api.services.crypto;

import com.nimbusds.jose.jwk.RSAKey;
import de.fitko.fitconnect.api.domain.model.metadata.data.Data;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.Attachment;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionSubmit;
import de.fitko.fitconnect.api.exceptions.DecryptionException;
import de.fitko.fitconnect.api.exceptions.EncryptionException;

/**
 * A service that allows to encrypt and decrypt {@link Data} and {@link Attachment}s of a {@link SubmissionSubmit}
 * via JSON-Web-Encryption.
 *
 * @see <a href="https://datatracker.ietf.org/doc/html/rfc7516">JSON-Web-Encryption</a>
 */
public interface CryptoService {

    /**
     * Decrypts a JWE encrypted string with the given private key.
     *
     * @param privateKey    RSA private key for decryption of JWE
     * @param encryptedData serialized encrypted JWE string
     * @return decrypted JWE string
     *
     * @throws DecryptionException if the payload cannot be decrypted or there was an issue with the key
     */
    String decryptString(RSAKey privateKey, String encryptedData) throws DecryptionException;

    /**
     * Decrypts a JWE encrypted string with the given private key.
     *
     * @param privateKey    RSA private key for decryption of JWE
     * @param encryptedData serialized encrypted JWE string
     * @return decrypted JWE byte array
     *
     * @throws DecryptionException if the payload cannot be decrypted or there was an issue with the key
     */
    byte[] decryptBytes(RSAKey privateKey, String encryptedData) throws DecryptionException;

    /**
     * Encrypts a string with the given public key.
     *
     * @param publicKey RSA public key for encryption of string payload
     * @param data      json or xml data that should be encrypted
     * @return a string serialization of the encrypted JWE object
     *
     * @throws EncryptionException if the payload cannot be encrypted or there was an issue with the key
     */
    String encryptString(RSAKey publicKey, String data) throws EncryptionException;

    /**
     * Encrypts a byte[] payload with the given public key.
     *
     * @param publicKey RSA public key the payload is encrypted with
     * @param bytes     byte[] of the data that should be encrypted
     * @return a string serialization of the encrypted JWE object
     *
     * @throws EncryptionException if the payload cannot be encrypted or there was an issue with the key
     */
    String encryptBytes(RSAKey publicKey, byte[] bytes) throws EncryptionException;
}
